@@ -1,7 +1,6 @@
 /*============================================================================
-
-  WCSLIB 4.17 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2013, Mark Calabretta
+  WCSLIB 7.11 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2022, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -18,28 +17,27 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: fitshdr.h,v 4.17 2013/01/29 05:29:20 cal103 Exp $
+  $Id: fitshdr.h,v 7.11 2022/04/26 06:13:52 mcalabre Exp $
 *=============================================================================
 *
-* The Flexible Image Transport System (FITS), a data format widely used in
-* astronomy for data interchange and archive, is described in
-*
-*   "Definition of The Flexible Image Transport System (FITS)",
-*   Hanisch, R.J., Farris, A., Greisen, E.W., et al. 2001, A&A, 376, 359
-*
-* which formalizes NOST 100-2.0, a document produced by the NASA/Science
-* Office of Standards and Technology, see http://fits.gsfc.nasa.gov.
-*
-* Refer to the README file provided with WCSLIB for an overview of the
-* library.
+* WCSLIB 7.11 - C routines that implement the FITS World Coordinate System
+* (WCS) standard.  Refer to the README file provided with WCSLIB for an
+* overview of the library.
 *
 *
 * Summary of the fitshdr routines
 * -------------------------------
+* The Flexible Image Transport System (FITS), is a data format widely used in
+* astronomy for data interchange and archive.  It is described in
+*
+=   "Definition of the Flexible Image Transport System (FITS), version 3.0",
+=   Pence, W.D., Chiappetti, L., Page, C.G., Shaw, R.A., & Stobie, E. 2010,
+=   A&A, 524, A42 - http://dx.doi.org/10.1051/0004-6361/201015362
+*
+* See also http://fits.gsfc.nasa.gov
+*
 * fitshdr() is a generic FITS header parser provided to handle keyrecords that
 * are ignored by the WCS header parsers, wcspih() and wcsbth().  Typically the
 * latter may be set to remove WCS keyrecords from a header leaving fitshdr()
@@ -73,7 +71,7 @@
 *                       keyids[] provides a convienient way of indexing them.
 *                       The fitskeyid struct contains three members;
 *                       fitskeyid::name must be set by the user while
-*                       fitskeyid::count and fitskeyid::name are returned by
+*                       fitskeyid::count and fitskeyid::idx are returned by
 *                       fitshdr().  All matched keywords will have their
 *                       fitskey::keyno member negated.
 *
@@ -87,8 +85,7 @@
 *                       the header.
 *
 *                       Memory for the array is allocated by fitshdr() and
-*                       this must be freed by the user by invoking free() on
-*                       the array.
+*                       this must be freed by the user.  See wcsdealloc().
 *
 * Function return value:
 *             int       Status return value:
@@ -96,6 +93,7 @@
 *                         1: Null fitskey pointer passed.
 *                         2: Memory allocation failed.
 *                         3: Fatal error returned by Flex parser.
+*                         4: Unrecognised data type.
 *
 * Notes:
 *   1: Keyword parsing is done in accordance with the syntax defined by
@@ -133,7 +131,7 @@
 *
 *      f: Sect. 5.2.3 offers no comment on the size of an integer keyvalue
 *         except indirectly in limiting it to 70 digits.  The parser will
-*         translates an integer keyvalue to a 32-bit signed integer if it
+*         translate an integer keyvalue to a 32-bit signed integer if it
 *         lies in the range -2147483648 to +2147483647, otherwise it
 *         interprets it as a 64-bit signed integer if possible, or else a
 *         "very long" integer (see fitskey::type).
@@ -249,7 +247,7 @@
 *
 *   int type
 *     (Returned) Keyvalue data type:
-*       - 0: No keyvalue.
+*       - 0: No keyvalue (both the value and type are undefined).
 *       - 1: Logical, represented as int.
 *       - 2: 32-bit signed integer.
 *       - 3: 64-bit signed integer (see below).
@@ -384,12 +382,20 @@ extern "C" {
 #define FITSHDR_KEYVALUE 0x02
 #define FITSHDR_COMMENT  0x04
 #define FITSHDR_KEYREC   0x08
-#define FITSHDR_CARD     0x08	/* Alias for backwards compatibility. */
+#define FITSHDR_CARD     0x08	// Alias for backwards compatibility.
 #define FITSHDR_TRAILER  0x10
 
 
 extern const char *fitshdr_errmsg[];
 
+enum fitshdr_errmsg_enum {
+  FITSHDRERR_SUCCESS      = 0,	// Success.
+  FITSHDRERR_NULL_POINTER = 1,	// Null fitskey pointer passed.
+  FITSHDRERR_MEMORY       = 2,	// Memory allocation failed.
+  FITSHDRERR_FLEX_PARSER  = 3,	// Fatal error returned by Flex parser.
+  FITSHDRERR_DATA_TYPE    = 4 	// Unrecognised data type.
+};
+
 #ifdef WCSLIB_INT64
   typedef WCSLIB_INT64 int64;
 #else
@@ -397,38 +403,38 @@ extern const char *fitshdr_errmsg[];
 #endif
 
 
-/* Struct used for indexing the keywords. */
+// Struct used for indexing the keywords.
 struct fitskeyid {
-  char name[12];		/* Keyword name, null-terminated.           */
-  int  count;			/* Number of occurrences of keyword.        */
-  int  idx[2];			/* Indices into fitskey array.              */
+  char name[12];		// Keyword name, null-terminated.
+  int  count;			// Number of occurrences of keyword.
+  int  idx[2];			// Indices into fitskey array.
 };
 
-/* Size of the fitskeyid struct in int units, used by the Fortran wrappers. */
+// Size of the fitskeyid struct in int units, used by the Fortran wrappers.
 #define KEYIDLEN (sizeof(struct fitskeyid)/sizeof(int))
 
 
-/* Struct used for storing FITS keywords. */
+// Struct used for storing FITS keywords.
 struct fitskey {
-  int  keyno;			/* Header keyrecord sequence number (1-rel).*/
-  int  keyid;			/* Index into fitskeyid[].                  */
-  int  status;			/* Header keyrecord status bit flags.       */
-  char keyword[12];		/* Keyword name, null-filled.               */
-  int  type;			/* Keyvalue type (see above).               */
-  int  padding;			/* (Dummy inserted for alignment purposes.) */
+  int  keyno;			// Header keyrecord sequence number (1-rel).
+  int  keyid;			// Index into fitskeyid[].
+  int  status;			// Header keyrecord status bit flags.
+  char keyword[12];		// Keyword name, null-filled.
+  int  type;			// Keyvalue type (see above).
+  int  padding;			// (Dummy inserted for alignment purposes.)
   union {
-    int    i;			/* 32-bit integer and logical values.       */
-    int64  k;			/* 64-bit integer values.                   */
-    int    l[8];		/* Very long signed integer values.         */
-    double f;			/* Floating point values.                   */
-    double c[2];		/* Complex values.                          */
-    char   s[72];		/* String values, null-terminated.          */
-  } keyvalue;			/* Keyvalue.                                */
-  int  ulen;			/* Length of units string.                  */
-  char comment[84];		/* Comment (or keyrecord), null-terminated. */
+    int    i;			// 32-bit integer and logical values.
+    int64  k;			// 64-bit integer values.
+    int    l[8];		// Very long signed integer values.
+    double f;			// Floating point values.
+    double c[2];		// Complex values.
+    char   s[72];		// String values, null-terminated.
+  } keyvalue;			// Keyvalue.
+  int  ulen;			// Length of units string.
+  char comment[84];		// Comment (or keyrecord), null-terminated.
 };
 
-/* Size of the fitskey struct in int units, used by the Fortran wrappers. */
+// Size of the fitskey struct in int units, used by the Fortran wrappers.
 #define KEYLEN (sizeof(struct fitskey)/sizeof(int))
 
 
@@ -440,4 +446,4 @@ int fitshdr(const char header[], int nkeyrec, int nkeyids,
 }
 #endif
 
-#endif /* WCSLIB_FITSHDR */
+#endif // WCSLIB_FITSHDR
